@@ -84,6 +84,8 @@ def check_rcfile():
         for s in ('rhythm', 'rhythmtapping2'):
             cfg.del_key("%s/countin_perc" % s)
             cfg.del_key("%s/rhythm_perc" % s)
+    if cfg.get_int("app/rcfileversion") <= 12:
+        cfg.del_key("sound/card_info")
     cfg.set_int("app/rcfileversion", rcfileversion)
     try:
         a = mpd.notename_to_int(cfg.get_string("user/lowest_pitch"))
@@ -166,14 +168,10 @@ class SolfegeApp(cfg.ConfigUtils):
             self.m_ui.display_error_message(
 _("You should configure sound from the 'Sound' page of the preferences window."))
         elif cfg.get_string("sound/type") == "sequencer-device":
-            if not buildinfo.HAVE_LINUX_AWE_VOICE_H and \
-                    cfg.get_string("sound/card_info") == "awe":
-                cfg.set_string("sound/card_info", "");
             try:
                 soundcard.initialise_devicefile(
                              cfg.get_string("sound/device_file"),
                              cfg.get_int("sound/synth_number"),
-                             cfg.get_string("sound/card_info"),
                              verbose_init=self.m_options.verbose_sound_init)
             except (soundcard.SoundInitException, OSError, ImportError), e:
                 self.m_sound_init_exception = e
