@@ -17,9 +17,9 @@
 import sys
 from _version import version_info
 
-VERSION_STRING = '3.11.0'
-REVISION_ID = 'tca@gnu.org-20080323173741-joy4m6j0gzut4ky0'
-HAVE_LINUX_AWE_VOICE_H = "yes" == "yes"
+VERSION_STRING = '3.11.1'
+REVISION_ID = 'tca@gnu.org-20080608162119-zgmd06fdxmhtvue7'
+HAVE_LINUX_AWE_VOICE_H = "@HAVE_LINUX_AWE_VOICE_H@" == "yes"
 ENABLE_TUNER = "no" == "yes"
 prefix = "/usr/local"
 
