@@ -225,10 +225,11 @@ lessonfile_functions = {
         'pangomarkup': (False, lambda s: LabelObject("pangomarkup", s)),
         'plabel': (False, lambda a, b, c: LabelObject("plabel", (a, b, c))),
         'rnc': (False, lambda s: LabelObject("rnc", s)),
+        'chordname': (False, lambda s: LabelObject("chordname", s)),
 }
 
 
-def rn_markup_tokenizer(s):
+def rnc_markup_tokenizer(s):
     """
     [rn][mod1][num][\s-]
     """
@@ -239,14 +240,56 @@ def rn_markup_tokenizer(s):
                        re.VERBOSE)
     i = 0
     retval = []
-    while True:
+    while i < len(s):
         m = rn_re.match(s[i:])
         if not m:
+            retval.append((u'ERR:%s' % s[i:], '', '', ''))
             break
         retval.append((m.group('p1'), m.group('p2'), m.group('p3'), m.group('sep')))
         i += m.end()
     return retval
 
+def chordname_markup_tokenizer(s):
+    """
+    s argument is a string of white-space separated chords.
+    Each chord is in the following format
+    [notename][whatever-text-in-normal-font-size]:[whatever-text-in-superscript-font][/notename]
+    [notename] is a solfege internal format notename, c cis des fis bes etc...
+    c
+    caug    (allow c+ ??)
+    cm:9
+    c:11b9/g
+    """
+    r = re.compile("""(?P<nn>[cdefgab](es|is)*)
+                (?P<txt1>.*?)
+                (:
+                 (?P<sup>.*?))?
+                $
+                """, re.VERBOSE)
+    retval = []
+    for c in s.split():
+        v = c.split("/")
+        if len(v) > 1:
+            try:
+                mpd.MusicalPitch.new_from_notename(v[-1])
+                bass = v[-1]
+                c = "/".join(v[:-1])
+            except mpd.InvalidNotenameException, e:
+                bass = ""
+        else:
+            bass = ""
+        m = r.match(c)
+        if m:
+            txt1 = m.group('txt1')
+            if not txt1:
+                txt1 = ""
+            sup = m.group('sup')
+            if not sup:
+                sup = ""
+            retval.append((m.group('nn'), txt1, sup, bass))
+        else:
+            retval.append(('ERR', '', '', ''))
+    return retval
 
 class _Header(dict):
     def __init__(self, headerdict):
@@ -1450,7 +1493,6 @@ class LessonFileManager:
         self.m_uiddb = {}
         for dir in lessonpath:
             if not os.path.isdir(dir):
-                print "warning: invalid directory in path:", dir
                 continue
             v = glob.glob(os.path.join(dir, "*"))
             for filename in v:
