@@ -191,6 +191,7 @@ import mpd
 import mpd.musicdisplayer
 import gu
 import cfg
+import stock
 import lessonfile
 import lessonfilegui
 import abstract
@@ -248,6 +249,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self._vbox = gtk.VBox()
         self._vbox.show()
         self.add(self._vbox)
+        stock.SolfegeIconFactory(self, datadir)
         self.set_default_size(400, 400)
         pixbuf = self.render_icon('solfege-icon', gtk.ICON_SIZE_DIALOG)
         self.set_icon(pixbuf)
@@ -268,7 +270,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self.m_app = app.SolfegeApp(options, self, lessonfile_manager)
         self.g_ui_manager = gtk.UIManager()
         self.m_action_groups = {
-            'Practise': gtk.ActionGroup('Practise'),
+            'Exercises': gtk.ActionGroup('Exercises'),
             'Exit': gtk.ActionGroup('Exit'),
             'NotExit': gtk.ActionGroup('NotExit'),
         }
@@ -372,24 +374,36 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         if self.m_learning_trees_merge_id:
             self.g_ui_manager.remove_ui(self.m_learning_trees_merge_id)
         trees = learning_tree_editor.get_learning_tree_list(self.m_app.m_options.debug)
-        actions = [('LearningTreesMenu', None, _('Learning tree'))]
+        if 'LearningTreesMenu' not in [a.get_name() for a in self.m_action_groups['NotExit'].list_actions()]:
+            actions = [('LearningTreesMenu', None, _('Learning tree'))]
+        else:
+            actions = []
         s = """
 <menubar name='Menubar'>
  <menu action='FileMenu'>
   <menu action='LearningTreesMenu' position='top'>\n"""
         old_cls = trees[0][0]
         for cls, fn, fullpath in trees:
+            # FIXME: the separator seems to be invisible most of the time. Why???
             if old_cls != cls:
                 s += "<separator/>\n"
                 old_cls = cls
             t = learning_tree_editor.LearningTree(self.m_app.lessonfile_manager)
-            if cls == 'solfege':
-                t.read_file(os.path.join("learningtrees", fn))
-            else:
-                t.read_file(os.path.join(os.path.join(filesystem.user_data(), 'learningtrees', fn)))
+            try:
+                t.read_file(fullpath)
+            except Exception, e:
+                if splash_win:
+                    splash_win.hide()
+                gu.dialog_ok(_("Failed to load learning tree"), self, 
+                    _("Ignoring the file \"%(filename(s\" because of this error: %(error)s") % {
+                    'filename': fullpath,
+                    'error': str(e).decode(sys.getfilesystemencoding(), 'replace'),
+                    }, gtk.MESSAGE_ERROR)
+                continue
             s += "  <menuitem action='%s:%s'/>\n" % (cls, fn)
-            actions.append(('%s:%s' % (cls, fn), None, _(t.m_title),
-                None, None,
+            action_name = '%s:%s' % (cls, fn)
+            if action_name not in [x.get_name() for x in self.m_action_groups['NotExit'].list_actions()]:
+                actions.append((action_name, None, _(t.m_title), None, None,
                 lambda o, c=cls, f=fn: self.change_learning_tree((c, f))))
         s += "</menu></menu></menubar>"
         self.m_action_groups['NotExit'].add_actions(actions)
@@ -406,45 +420,60 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         if self.m_practise_and_test_merge_id:
             self.g_ui_manager.remove_ui(self.m_practise_and_test_merge_id)
             self.g_ui_manager.ensure_update()
-        s = "<menubar name='Menubar'>"
+        # This method can be called more than once. So we must first
+        # remove any actions from the last time it was called.
+        for action in self.m_action_groups['Exercises'].list_actions():
+            self.m_action_groups['Exercises'].remove_action(action)
+        # We use this counter to make sure all Action names are unique.
+        # This to avoid warnings if a toplevel menu and a submenu has
+        # the same name
+        id_counter = 0
+        s = "<menubar name='Menubar'><placeholder name='ExercisesPlaceHolder'>"
         tests = "<menu action='TestsMenu'>"
         for menu in self.m_tree.m_menus:
-            self.m_action_groups['NotExit'].add_action(
-                gtk.Action(menu.name, _(menu.name), None, None))
-            s += "<menu action='%s'>" % escape(menu.name)
+            id_counter += 1
+            self.m_action_groups['Exercises'].add_action(
+                gtk.Action("%s%i" % (menu.name, id_counter),
+                           _(menu.name), None, None))
+            s += "<menu action='%s%i'>" % (escape(menu.name), id_counter)
             # The following line might look strange here, but it makes sure
             # that the menus on the menu bar stay at the correct order after
             # we save the learning tree in the learning tree editor.
-            self.g_ui_manager.add_ui_from_string("<menubar name='Menubar'><menu action='%s'/></menubar>" % escape(menu.name))
+            self.g_ui_manager.add_ui_from_string("<menubar name='Menubar'><menu action='%s%i'/></menubar>" % (escape(menu.name), id_counter))
             for topic in menu.children:
-                s += "<menu action='%s'>" % escape(topic['name'])
-                tests += "<menu action='TEST_%s'>" % escape(topic['name'])
-                actions.append((topic['name'], None, _(topic['name'])))
-                actions.append(('TEST_%s' % topic['name'], None, _(topic['name'])))
+                id_counter += 1
+                s += "<menu action='%s%i'>" % (escape(topic['name']), id_counter)
+                tests += "<menu action='TEST_%s%i'>" % (escape(topic['name']), id_counter)
+                actions.append(("%s%i" % (topic['name'], id_counter),
+                                None, _(topic['name'])))
+                actions.append(('TEST_%s%i' % (topic['name'], id_counter),
+                               None, _(topic['name'])))
                 for lesson_id in topic.children:
                     if self.m_tree.m_visibilities[lesson_id] > self.m_tree.m_visibility:
                         continue
                     menutitle = self.m_app.lessonfile_manager.m_uiddb[lesson_id]['header']['title']
+                    id_counter += 1
                     actions.append((
-                        lesson_id, None, menutitle, None, None,
+                        "%s%i" % (lesson_id, id_counter),
+                        None, menutitle, None, None,
                         lambda o, e=lesson_id: self.m_app.practise_lesson_id(e)))
                     if self.m_app.lessonfile_manager.is_test_passed(lesson_id):
                         menutitle = _("%s (passed)") % menutitle
                     if self.m_app.lessonfile_manager.get(lesson_id, 'test'):
                         actions.append((
-                            'TEST_%s' % lesson_id, None,
+                            'TEST_%s%s' % (lesson_id, id_counter), None,
                             menutitle,
                             None, None,
                             lambda o, e=lesson_id: self.m_app.test_lesson_id(e)))
-                        tests += "<menuitem action='TEST_%s' />" % escape(lesson_id)
-                    s += "<menuitem action='%s' />" % escape(lesson_id)
+                        tests += "<menuitem action='TEST_%s%i' />" % (escape(lesson_id), id_counter)
+                    s += "<menuitem action='%s%i' />" % (escape(lesson_id), id_counter)
                 s += "</menu>"
                 tests += "</menu>"
             s += "</menu>"
-        self.m_action_groups['NotExit'].add_actions(actions)
+        self.m_action_groups['Exercises'].add_actions(actions)
         tests += "</menu>"
         s += tests
-        s += "</menubar>"
+        s += "</placeholder></menubar>"
         # The following line might look strange here, but it makes sure
         # that the menus on the menu bar stay at the correct order after
         # we save the learning tree in the learning tree editor.
@@ -673,6 +702,24 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             self.display_discarded_lessonfiles_info()
             if splash_win:
                 splash_win.show()
+        # MIGRATION 3.11.1: earlier editors would create new learning trees
+        # below app_data() instead of user_data().
+        if (sys.platform == "win32" and 
+            os.path.exists(os.path.join(filesystem.app_data(),
+                                        "learningtrees"))):
+            if not os.path.exists(os.path.join(filesystem.user_data(), "learningtrees")):
+                os.mkdir(os.path.join(filesystem.user_data(), "learningtrees"))
+            for fn in os.listdir(os.path.join(filesystem.app_data(), "learningtrees")):
+                if not os.path.exists(os.path.join(filesystem.user_data(), "learningtrees", fn)):
+                    shutil.move(os.path.join(filesystem.app_data(), "learningtrees", fn),
+                            os.path.join(filesystem.user_data(), "learningtrees"))
+                else:
+                    # We add the .bak exstention if the file already exists.
+                    shutil.move(os.path.join(filesystem.app_data(), "learningtrees", fn),
+                            os.path.join(filesystem.user_data(), "learningtrees", u"%s.bak" % fn))
+                os.rmdir(os.path.join(os.path.join(filesystem.app_data(), "learningtrees")))
+        item = self.g_ui_manager.get_widget("/Menubar/FileMenu/LearningTreesMenu")
+        item.connect('activate', lambda s: self.create_learning_trees_menu())
     def display_discarded_lessonfiles_info(self):
         assert self.m_app.lessonfile_manager.m_discards
         dlg = gtk.Dialog(_("Discarded lesson files"), buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
@@ -815,6 +862,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             gu.dialog_ok(_("The '%s' exercise module does not support test yet." % self.m_viewer))
             return
         self.m_action_groups['NotExit'].set_sensitive(False)
+        self.m_action_groups['Exercises'].set_sensitive(False)
         self.g = self.box_dict[self.m_viewer].g_notebook.get_nth_page(0)
         self.box_dict[self.m_viewer].g_notebook.get_nth_page(0).reparent(self.main_box)
         self.box_dict[self.m_viewer].g_notebook.hide()
@@ -822,6 +870,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
     def exit_test_mode(self):
         self.m_app.m_test_mode = False
         self.m_action_groups['NotExit'].set_sensitive(True)
+        self.m_action_groups['Exercises'].set_sensitive(True)
         box = gtk.VBox()
         self.box_dict[self.m_viewer].g_notebook.insert_page(box, gtk.Label(_("Practise")), 0)
         self.g.reparent(box)
